<?php
/*
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * GENERATED CODE WARNING
 * Generated by gapic-generator-php from the file
 * https://github.com/googleapis/googleapis/blob/master/google/cloud/compute/v1/compute.proto
 * Updates to the above are reflected here through a refresh process.
 */

namespace Google\Cloud\Compute\V1\Gapic;

use Google\ApiCore\ApiException;
use Google\ApiCore\CredentialsWrapper;
use Google\ApiCore\GapicClientTrait;
use Google\ApiCore\OperationResponse;
use Google\ApiCore\RequestParamsHeaderDescriptor;
use Google\ApiCore\RetrySettings;
use Google\ApiCore\Transport\TransportInterface;
use Google\ApiCore\ValidationException;
use Google\Auth\FetchAuthTokenInterface;
use Google\Cloud\Compute\V1\AddAssociationNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\AddRuleNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\CloneRulesNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\DeleteNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\FirewallPolicy;
use Google\Cloud\Compute\V1\FirewallPolicyAssociation;
use Google\Cloud\Compute\V1\FirewallPolicyList;
use Google\Cloud\Compute\V1\FirewallPolicyRule;
use Google\Cloud\Compute\V1\GetAssociationNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\GetIamPolicyNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\GetNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\GetRuleNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\GlobalOperationsClient;
use Google\Cloud\Compute\V1\GlobalSetPolicyRequest;
use Google\Cloud\Compute\V1\InsertNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\ListNetworkFirewallPoliciesRequest;
use Google\Cloud\Compute\V1\Operation;
use Google\Cloud\Compute\V1\PatchNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\PatchRuleNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\Policy;
use Google\Cloud\Compute\V1\RemoveAssociationNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\RemoveRuleNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\SetIamPolicyNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\TestIamPermissionsNetworkFirewallPolicyRequest;
use Google\Cloud\Compute\V1\TestPermissionsRequest;
use Google\Cloud\Compute\V1\TestPermissionsResponse;

/**
 * Service Description: The NetworkFirewallPolicies API.
 *
 * This class provides the ability to make remote calls to the backing service through method
 * calls that map to API methods. Sample code to get started:
 *
 * ```
 * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
 * try {
 *     $firewallPolicy = 'firewall_policy';
 *     $firewallPolicyAssociationResource = new FirewallPolicyAssociation();
 *     $project = 'project';
 *     $operationResponse = $networkFirewallPoliciesClient->addAssociation($firewallPolicy, $firewallPolicyAssociationResource, $project);
 *     $operationResponse->pollUntilComplete();
 *     if ($operationResponse->operationSucceeded()) {
 *         // if creating/modifying, retrieve the target resource
 *     } else {
 *         $error = $operationResponse->getError();
 *         // handleError($error)
 *     }
 *     // Alternatively:
 *     // start the operation, keep the operation name, and resume later
 *     $operationResponse = $networkFirewallPoliciesClient->addAssociation($firewallPolicy, $firewallPolicyAssociationResource, $project);
 *     $operationName = $operationResponse->getName();
 *     // ... do other work
 *     $newOperationResponse = $networkFirewallPoliciesClient->resumeOperation($operationName, 'addAssociation');
 *     while (!$newOperationResponse->isDone()) {
 *         // ... do other work
 *         $newOperationResponse->reload();
 *     }
 *     if ($newOperationResponse->operationSucceeded()) {
 *         // if creating/modifying, retrieve the target resource
 *     } else {
 *         $error = $newOperationResponse->getError();
 *         // handleError($error)
 *     }
 * } finally {
 *     $networkFirewallPoliciesClient->close();
 * }
 * ```
 */
class NetworkFirewallPoliciesGapicClient
{
    use GapicClientTrait;

    /** The name of the service. */
    const SERVICE_NAME = 'google.cloud.compute.v1.NetworkFirewallPolicies';

    /** The default address of the service. */
    const SERVICE_ADDRESS = 'compute.googleapis.com';

    /** The default port of the service. */
    const DEFAULT_SERVICE_PORT = 443;

    /** The name of the code generator, to be included in the agent header. */
    const CODEGEN_NAME = 'gapic';

    /** The default scopes required by the service. */
    public static $serviceScopes = [
        'https://www.googleapis.com/auth/compute',
        'https://www.googleapis.com/auth/cloud-platform',
    ];

    private $operationsClient;

    private static function getClientDefaults()
    {
        return [
            'serviceName' => self::SERVICE_NAME,
            'apiEndpoint' => self::SERVICE_ADDRESS . ':' . self::DEFAULT_SERVICE_PORT,
            'clientConfig' => __DIR__ . '/../resources/network_firewall_policies_client_config.json',
            'descriptorsConfigPath' => __DIR__ . '/../resources/network_firewall_policies_descriptor_config.php',
            'credentialsConfig' => [
                'defaultScopes' => self::$serviceScopes,
                'useJwtAccessWithScope' => false,
            ],
            'transportConfig' => [
                'rest' => [
                    'restClientConfigPath' => __DIR__ . '/../resources/network_firewall_policies_rest_client_config.php',
                ],
            ],
            'operationsClientClass' => GlobalOperationsClient::class,
        ];
    }

    /** Implements GapicClientTrait::defaultTransport. */
    private static function defaultTransport()
    {
        return 'rest';
    }

    /** Implements GapicClientTrait::getSupportedTransports. */
    private static function getSupportedTransports()
    {
        return [
            'rest',
        ];
    }

    /**
     * Return an GlobalOperationsClient object with the same endpoint as $this.
     *
     * @return GlobalOperationsClient
     */
    public function getOperationsClient()
    {
        return $this->operationsClient;
    }

    /** Return the default longrunning operation descriptor config. */
    private function getDefaultOperationDescriptor()
    {
        return [
            'additionalArgumentMethods' => [
                'getProject',
            ],
            'getOperationMethod' => 'get',
            'cancelOperationMethod' => null,
            'deleteOperationMethod' => 'delete',
            'operationErrorCodeMethod' => 'getHttpErrorStatusCode',
            'operationErrorMessageMethod' => 'getHttpErrorMessage',
            'operationNameMethod' => 'getName',
            'operationStatusMethod' => 'getStatus',
            'operationStatusDoneValue' => \Google\Cloud\Compute\V1\Operation\Status::DONE,
        ];
    }

    /**
     * Resume an existing long running operation that was previously started by a long
     * running API method. If $methodName is not provided, or does not match a long
     * running API method, then the operation can still be resumed, but the
     * OperationResponse object will not deserialize the final response.
     *
     * @param string $operationName The name of the long running operation
     * @param string $methodName    The name of the method used to start the operation
     *
     * @return OperationResponse
     */
    public function resumeOperation($operationName, $methodName = null)
    {
        $options = isset($this->descriptors[$methodName]['longRunning']) ? $this->descriptors[$methodName]['longRunning'] : $this->getDefaultOperationDescriptor();
        $operation = new OperationResponse($operationName, $this->getOperationsClient(), $options);
        $operation->reload();
        return $operation;
    }

    /**
     * Constructor.
     *
     * @param array $options {
     *     Optional. Options for configuring the service API wrapper.
     *
     *     @type string $apiEndpoint
     *           The address of the API remote host. May optionally include the port, formatted
     *           as "<uri>:<port>". Default 'compute.googleapis.com:443'.
     *     @type string|array|FetchAuthTokenInterface|CredentialsWrapper $credentials
     *           The credentials to be used by the client to authorize API calls. This option
     *           accepts either a path to a credentials file, or a decoded credentials file as a
     *           PHP array.
     *           *Advanced usage*: In addition, this option can also accept a pre-constructed
     *           {@see \Google\Auth\FetchAuthTokenInterface} object or
     *           {@see \Google\ApiCore\CredentialsWrapper} object. Note that when one of these
     *           objects are provided, any settings in $credentialsConfig will be ignored.
     *     @type array $credentialsConfig
     *           Options used to configure credentials, including auth token caching, for the
     *           client. For a full list of supporting configuration options, see
     *           {@see \Google\ApiCore\CredentialsWrapper::build()} .
     *     @type bool $disableRetries
     *           Determines whether or not retries defined by the client configuration should be
     *           disabled. Defaults to `false`.
     *     @type string|array $clientConfig
     *           Client method configuration, including retry settings. This option can be either
     *           a path to a JSON file, or a PHP array containing the decoded JSON data. By
     *           default this settings points to the default client config file, which is
     *           provided in the resources folder.
     *     @type string|TransportInterface $transport
     *           The transport used for executing network requests. At the moment, supports only
     *           `rest`. *Advanced usage*: Additionally, it is possible to pass in an already
     *           instantiated {@see \Google\ApiCore\Transport\TransportInterface} object. Note
     *           that when this object is provided, any settings in $transportConfig, and any
     *           $apiEndpoint setting, will be ignored.
     *     @type array $transportConfig
     *           Configuration options that will be used to construct the transport. Options for
     *           each supported transport type should be passed in a key for that transport. For
     *           example:
     *           $transportConfig = [
     *               'rest' => [...],
     *           ];
     *           See the {@see \Google\ApiCore\Transport\RestTransport::build()} method for the
     *           supported options.
     *     @type callable $clientCertSource
     *           A callable which returns the client cert as a string. This can be used to
     *           provide a certificate and private key to the transport layer for mTLS.
     * }
     *
     * @throws ValidationException
     */
    public function __construct(array $options = [])
    {
        $clientOptions = $this->buildClientOptions($options);
        $this->setClientOptions($clientOptions);
        $this->operationsClient = $this->createOperationsClient($clientOptions);
    }

    /**
     * Inserts an association for the specified firewall policy.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $firewallPolicyAssociationResource = new FirewallPolicyAssociation();
     *     $project = 'project';
     *     $operationResponse = $networkFirewallPoliciesClient->addAssociation($firewallPolicy, $firewallPolicyAssociationResource, $project);
     *     $operationResponse->pollUntilComplete();
     *     if ($operationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $operationResponse->getError();
     *         // handleError($error)
     *     }
     *     // Alternatively:
     *     // start the operation, keep the operation name, and resume later
     *     $operationResponse = $networkFirewallPoliciesClient->addAssociation($firewallPolicy, $firewallPolicyAssociationResource, $project);
     *     $operationName = $operationResponse->getName();
     *     // ... do other work
     *     $newOperationResponse = $networkFirewallPoliciesClient->resumeOperation($operationName, 'addAssociation');
     *     while (!$newOperationResponse->isDone()) {
     *         // ... do other work
     *         $newOperationResponse->reload();
     *     }
     *     if ($newOperationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $newOperationResponse->getError();
     *         // handleError($error)
     *     }
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string                    $firewallPolicy                    Name of the firewall policy to update.
     * @param FirewallPolicyAssociation $firewallPolicyAssociationResource The body resource for this request
     * @param string                    $project                           Project ID for this request.
     * @param array                     $optionalArgs                      {
     *     Optional.
     *
     *     @type bool $replaceExistingAssociation
     *           Indicates whether or not to replace it if an association of the attachment already exists. This is false by default, in which case an error will be returned if an association already exists.
     *     @type string $requestId
     *           An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\OperationResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function addAssociation($firewallPolicy, $firewallPolicyAssociationResource, $project, array $optionalArgs = [])
    {
        $request = new AddAssociationNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setFirewallPolicyAssociationResource($firewallPolicyAssociationResource);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['replaceExistingAssociation'])) {
            $request->setReplaceExistingAssociation($optionalArgs['replaceExistingAssociation']);
        }

        if (isset($optionalArgs['requestId'])) {
            $request->setRequestId($optionalArgs['requestId']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startOperationsCall('AddAssociation', $optionalArgs, $request, $this->getOperationsClient(), null, Operation::class)->wait();
    }

    /**
     * Inserts a rule into a firewall policy.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $firewallPolicyRuleResource = new FirewallPolicyRule();
     *     $project = 'project';
     *     $operationResponse = $networkFirewallPoliciesClient->addRule($firewallPolicy, $firewallPolicyRuleResource, $project);
     *     $operationResponse->pollUntilComplete();
     *     if ($operationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $operationResponse->getError();
     *         // handleError($error)
     *     }
     *     // Alternatively:
     *     // start the operation, keep the operation name, and resume later
     *     $operationResponse = $networkFirewallPoliciesClient->addRule($firewallPolicy, $firewallPolicyRuleResource, $project);
     *     $operationName = $operationResponse->getName();
     *     // ... do other work
     *     $newOperationResponse = $networkFirewallPoliciesClient->resumeOperation($operationName, 'addRule');
     *     while (!$newOperationResponse->isDone()) {
     *         // ... do other work
     *         $newOperationResponse->reload();
     *     }
     *     if ($newOperationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $newOperationResponse->getError();
     *         // handleError($error)
     *     }
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string             $firewallPolicy             Name of the firewall policy to update.
     * @param FirewallPolicyRule $firewallPolicyRuleResource The body resource for this request
     * @param string             $project                    Project ID for this request.
     * @param array              $optionalArgs               {
     *     Optional.
     *
     *     @type int $maxPriority
     *           When rule.priority is not specified, auto choose a unused priority between minPriority and maxPriority>. This field is exclusive with rule.priority.
     *     @type int $minPriority
     *           When rule.priority is not specified, auto choose a unused priority between minPriority and maxPriority>. This field is exclusive with rule.priority.
     *     @type string $requestId
     *           An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\OperationResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function addRule($firewallPolicy, $firewallPolicyRuleResource, $project, array $optionalArgs = [])
    {
        $request = new AddRuleNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setFirewallPolicyRuleResource($firewallPolicyRuleResource);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['maxPriority'])) {
            $request->setMaxPriority($optionalArgs['maxPriority']);
        }

        if (isset($optionalArgs['minPriority'])) {
            $request->setMinPriority($optionalArgs['minPriority']);
        }

        if (isset($optionalArgs['requestId'])) {
            $request->setRequestId($optionalArgs['requestId']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startOperationsCall('AddRule', $optionalArgs, $request, $this->getOperationsClient(), null, Operation::class)->wait();
    }

    /**
     * Copies rules to the specified firewall policy.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $project = 'project';
     *     $operationResponse = $networkFirewallPoliciesClient->cloneRules($firewallPolicy, $project);
     *     $operationResponse->pollUntilComplete();
     *     if ($operationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $operationResponse->getError();
     *         // handleError($error)
     *     }
     *     // Alternatively:
     *     // start the operation, keep the operation name, and resume later
     *     $operationResponse = $networkFirewallPoliciesClient->cloneRules($firewallPolicy, $project);
     *     $operationName = $operationResponse->getName();
     *     // ... do other work
     *     $newOperationResponse = $networkFirewallPoliciesClient->resumeOperation($operationName, 'cloneRules');
     *     while (!$newOperationResponse->isDone()) {
     *         // ... do other work
     *         $newOperationResponse->reload();
     *     }
     *     if ($newOperationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $newOperationResponse->getError();
     *         // handleError($error)
     *     }
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string $firewallPolicy Name of the firewall policy to update.
     * @param string $project        Project ID for this request.
     * @param array  $optionalArgs   {
     *     Optional.
     *
     *     @type string $requestId
     *           An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
     *     @type string $sourceFirewallPolicy
     *           The firewall policy from which to copy rules.
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\OperationResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function cloneRules($firewallPolicy, $project, array $optionalArgs = [])
    {
        $request = new CloneRulesNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['requestId'])) {
            $request->setRequestId($optionalArgs['requestId']);
        }

        if (isset($optionalArgs['sourceFirewallPolicy'])) {
            $request->setSourceFirewallPolicy($optionalArgs['sourceFirewallPolicy']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startOperationsCall('CloneRules', $optionalArgs, $request, $this->getOperationsClient(), null, Operation::class)->wait();
    }

    /**
     * Deletes the specified policy.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $project = 'project';
     *     $operationResponse = $networkFirewallPoliciesClient->delete($firewallPolicy, $project);
     *     $operationResponse->pollUntilComplete();
     *     if ($operationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $operationResponse->getError();
     *         // handleError($error)
     *     }
     *     // Alternatively:
     *     // start the operation, keep the operation name, and resume later
     *     $operationResponse = $networkFirewallPoliciesClient->delete($firewallPolicy, $project);
     *     $operationName = $operationResponse->getName();
     *     // ... do other work
     *     $newOperationResponse = $networkFirewallPoliciesClient->resumeOperation($operationName, 'delete');
     *     while (!$newOperationResponse->isDone()) {
     *         // ... do other work
     *         $newOperationResponse->reload();
     *     }
     *     if ($newOperationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $newOperationResponse->getError();
     *         // handleError($error)
     *     }
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string $firewallPolicy Name of the firewall policy to delete.
     * @param string $project        Project ID for this request.
     * @param array  $optionalArgs   {
     *     Optional.
     *
     *     @type string $requestId
     *           An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\OperationResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function delete($firewallPolicy, $project, array $optionalArgs = [])
    {
        $request = new DeleteNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['requestId'])) {
            $request->setRequestId($optionalArgs['requestId']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startOperationsCall('Delete', $optionalArgs, $request, $this->getOperationsClient(), null, Operation::class)->wait();
    }

    /**
     * Returns the specified network firewall policy.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $project = 'project';
     *     $response = $networkFirewallPoliciesClient->get($firewallPolicy, $project);
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string $firewallPolicy Name of the firewall policy to get.
     * @param string $project        Project ID for this request.
     * @param array  $optionalArgs   {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\Compute\V1\FirewallPolicy
     *
     * @throws ApiException if the remote call fails
     */
    public function get($firewallPolicy, $project, array $optionalArgs = [])
    {
        $request = new GetNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('Get', FirewallPolicy::class, $optionalArgs, $request)->wait();
    }

    /**
     * Gets an association with the specified name.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $project = 'project';
     *     $response = $networkFirewallPoliciesClient->getAssociation($firewallPolicy, $project);
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string $firewallPolicy Name of the firewall policy to which the queried association belongs.
     * @param string $project        Project ID for this request.
     * @param array  $optionalArgs   {
     *     Optional.
     *
     *     @type string $name
     *           The name of the association to get from the firewall policy.
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\Compute\V1\FirewallPolicyAssociation
     *
     * @throws ApiException if the remote call fails
     */
    public function getAssociation($firewallPolicy, $project, array $optionalArgs = [])
    {
        $request = new GetAssociationNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['name'])) {
            $request->setName($optionalArgs['name']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('GetAssociation', FirewallPolicyAssociation::class, $optionalArgs, $request)->wait();
    }

    /**
     * Gets the access control policy for a resource. May be empty if no such policy or resource exists.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $project = 'project';
     *     $resource = 'resource';
     *     $response = $networkFirewallPoliciesClient->getIamPolicy($project, $resource);
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string $project      Project ID for this request.
     * @param string $resource     Name or id of the resource for this request.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type int $optionsRequestedPolicyVersion
     *           Requested IAM Policy version.
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\Compute\V1\Policy
     *
     * @throws ApiException if the remote call fails
     */
    public function getIamPolicy($project, $resource, array $optionalArgs = [])
    {
        $request = new GetIamPolicyNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setProject($project);
        $request->setResource($resource);
        $requestParamHeaders['project'] = $project;
        $requestParamHeaders['resource'] = $resource;
        if (isset($optionalArgs['optionsRequestedPolicyVersion'])) {
            $request->setOptionsRequestedPolicyVersion($optionalArgs['optionsRequestedPolicyVersion']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('GetIamPolicy', Policy::class, $optionalArgs, $request)->wait();
    }

    /**
     * Gets a rule of the specified priority.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $project = 'project';
     *     $response = $networkFirewallPoliciesClient->getRule($firewallPolicy, $project);
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string $firewallPolicy Name of the firewall policy to which the queried rule belongs.
     * @param string $project        Project ID for this request.
     * @param array  $optionalArgs   {
     *     Optional.
     *
     *     @type int $priority
     *           The priority of the rule to get from the firewall policy.
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\Compute\V1\FirewallPolicyRule
     *
     * @throws ApiException if the remote call fails
     */
    public function getRule($firewallPolicy, $project, array $optionalArgs = [])
    {
        $request = new GetRuleNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['priority'])) {
            $request->setPriority($optionalArgs['priority']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('GetRule', FirewallPolicyRule::class, $optionalArgs, $request)->wait();
    }

    /**
     * Creates a new policy in the specified project using the data included in the request.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicyResource = new FirewallPolicy();
     *     $project = 'project';
     *     $operationResponse = $networkFirewallPoliciesClient->insert($firewallPolicyResource, $project);
     *     $operationResponse->pollUntilComplete();
     *     if ($operationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $operationResponse->getError();
     *         // handleError($error)
     *     }
     *     // Alternatively:
     *     // start the operation, keep the operation name, and resume later
     *     $operationResponse = $networkFirewallPoliciesClient->insert($firewallPolicyResource, $project);
     *     $operationName = $operationResponse->getName();
     *     // ... do other work
     *     $newOperationResponse = $networkFirewallPoliciesClient->resumeOperation($operationName, 'insert');
     *     while (!$newOperationResponse->isDone()) {
     *         // ... do other work
     *         $newOperationResponse->reload();
     *     }
     *     if ($newOperationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $newOperationResponse->getError();
     *         // handleError($error)
     *     }
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param FirewallPolicy $firewallPolicyResource The body resource for this request
     * @param string         $project                Project ID for this request.
     * @param array          $optionalArgs           {
     *     Optional.
     *
     *     @type string $requestId
     *           An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\OperationResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function insert($firewallPolicyResource, $project, array $optionalArgs = [])
    {
        $request = new InsertNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicyResource($firewallPolicyResource);
        $request->setProject($project);
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['requestId'])) {
            $request->setRequestId($optionalArgs['requestId']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startOperationsCall('Insert', $optionalArgs, $request, $this->getOperationsClient(), null, Operation::class)->wait();
    }

    /**
     * Lists all the policies that have been configured for the specified project.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $project = 'project';
     *     // Iterate over pages of elements
     *     $pagedResponse = $networkFirewallPoliciesClient->list($project);
     *     foreach ($pagedResponse->iteratePages() as $page) {
     *         foreach ($page as $element) {
     *             // doSomethingWith($element);
     *         }
     *     }
     *     // Alternatively:
     *     // Iterate through all elements
     *     $pagedResponse = $networkFirewallPoliciesClient->list($project);
     *     foreach ($pagedResponse->iterateAllElements() as $element) {
     *         // doSomethingWith($element);
     *     }
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string $project      Project ID for this request.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type string $filter
     *           A filter expression that filters resources listed in the response. Most Compute resources support two types of filter expressions: expressions that support regular expressions and expressions that follow API improvement proposal AIP-160. If you want to use AIP-160, your expression must specify the field name, an operator, and the value that you want to use for filtering. The value must be a string, a number, or a boolean. The operator must be either `=`, `!=`, `>`, `<`, `<=`, `>=` or `:`. For example, if you are filtering Compute Engine instances, you can exclude instances named `example-instance` by specifying `name != example-instance`. The `:` operator can be used with string fields to match substrings. For non-string fields it is equivalent to the `=` operator. The `:*` comparison can be used to test whether a key has been defined. For example, to find all objects with `owner` label use: ``` labels.owner:* ``` You can also filter nested fields. For example, you could specify `scheduling.automaticRestart = false` to include instances only if they are not scheduled for automatic restarts. You can use filtering on nested fields to filter based on resource labels. To filter on multiple expressions, provide each separate expression within parentheses. For example: ``` (scheduling.automaticRestart = true) (cpuPlatform = "Intel Skylake") ``` By default, each expression is an `AND` expression. However, you can include `AND` and `OR` expressions explicitly. For example: ``` (cpuPlatform = "Intel Skylake") OR (cpuPlatform = "Intel Broadwell") AND (scheduling.automaticRestart = true) ``` If you want to use a regular expression, use the `eq` (equal) or `ne` (not equal) operator against a single un-parenthesized expression with or without quotes or against multiple parenthesized expressions. Examples: `fieldname eq unquoted literal` `fieldname eq 'single quoted literal'` `fieldname eq "double quoted literal"` `(fieldname1 eq literal) (fieldname2 ne "literal")` The literal value is interpreted as a regular expression using Google RE2 library syntax. The literal value must match the entire field. For example, to filter for instances that do not end with name "instance", you would use `name ne .*instance`.
     *     @type int $maxResults
     *           The maximum number of results per page that should be returned. If the number of available results is larger than `maxResults`, Compute Engine returns a `nextPageToken` that can be used to get the next page of results in subsequent list requests. Acceptable values are `0` to `500`, inclusive. (Default: `500`)
     *     @type string $orderBy
     *           Sorts list results by a certain order. By default, results are returned in alphanumerical order based on the resource name. You can also sort results in descending order based on the creation timestamp using `orderBy="creationTimestamp desc"`. This sorts results based on the `creationTimestamp` field in reverse chronological order (newest result first). Use this to sort resources like operations so that the newest operation is returned first. Currently, only sorting by `name` or `creationTimestamp desc` is supported.
     *     @type string $pageToken
     *           A page token is used to specify a page of values to be returned.
     *           If no page token is specified (the default), the first page
     *           of values will be returned. Any page token used here must have
     *           been generated by a previous call to the API.
     *     @type bool $returnPartialSuccess
     *           Opt-in for partial success behavior which provides partial results in case of failure. The default value is false.
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\PagedListResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function list($project, array $optionalArgs = [])
    {
        $request = new ListNetworkFirewallPoliciesRequest();
        $requestParamHeaders = [];
        $request->setProject($project);
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['filter'])) {
            $request->setFilter($optionalArgs['filter']);
        }

        if (isset($optionalArgs['maxResults'])) {
            $request->setMaxResults($optionalArgs['maxResults']);
        }

        if (isset($optionalArgs['orderBy'])) {
            $request->setOrderBy($optionalArgs['orderBy']);
        }

        if (isset($optionalArgs['pageToken'])) {
            $request->setPageToken($optionalArgs['pageToken']);
        }

        if (isset($optionalArgs['returnPartialSuccess'])) {
            $request->setReturnPartialSuccess($optionalArgs['returnPartialSuccess']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->getPagedListResponse('List', $optionalArgs, FirewallPolicyList::class, $request);
    }

    /**
     * Patches the specified policy with the data included in the request.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $firewallPolicyResource = new FirewallPolicy();
     *     $project = 'project';
     *     $operationResponse = $networkFirewallPoliciesClient->patch($firewallPolicy, $firewallPolicyResource, $project);
     *     $operationResponse->pollUntilComplete();
     *     if ($operationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $operationResponse->getError();
     *         // handleError($error)
     *     }
     *     // Alternatively:
     *     // start the operation, keep the operation name, and resume later
     *     $operationResponse = $networkFirewallPoliciesClient->patch($firewallPolicy, $firewallPolicyResource, $project);
     *     $operationName = $operationResponse->getName();
     *     // ... do other work
     *     $newOperationResponse = $networkFirewallPoliciesClient->resumeOperation($operationName, 'patch');
     *     while (!$newOperationResponse->isDone()) {
     *         // ... do other work
     *         $newOperationResponse->reload();
     *     }
     *     if ($newOperationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $newOperationResponse->getError();
     *         // handleError($error)
     *     }
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string         $firewallPolicy         Name of the firewall policy to update.
     * @param FirewallPolicy $firewallPolicyResource The body resource for this request
     * @param string         $project                Project ID for this request.
     * @param array          $optionalArgs           {
     *     Optional.
     *
     *     @type string $requestId
     *           An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\OperationResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function patch($firewallPolicy, $firewallPolicyResource, $project, array $optionalArgs = [])
    {
        $request = new PatchNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setFirewallPolicyResource($firewallPolicyResource);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['requestId'])) {
            $request->setRequestId($optionalArgs['requestId']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startOperationsCall('Patch', $optionalArgs, $request, $this->getOperationsClient(), null, Operation::class)->wait();
    }

    /**
     * Patches a rule of the specified priority.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $firewallPolicyRuleResource = new FirewallPolicyRule();
     *     $project = 'project';
     *     $operationResponse = $networkFirewallPoliciesClient->patchRule($firewallPolicy, $firewallPolicyRuleResource, $project);
     *     $operationResponse->pollUntilComplete();
     *     if ($operationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $operationResponse->getError();
     *         // handleError($error)
     *     }
     *     // Alternatively:
     *     // start the operation, keep the operation name, and resume later
     *     $operationResponse = $networkFirewallPoliciesClient->patchRule($firewallPolicy, $firewallPolicyRuleResource, $project);
     *     $operationName = $operationResponse->getName();
     *     // ... do other work
     *     $newOperationResponse = $networkFirewallPoliciesClient->resumeOperation($operationName, 'patchRule');
     *     while (!$newOperationResponse->isDone()) {
     *         // ... do other work
     *         $newOperationResponse->reload();
     *     }
     *     if ($newOperationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $newOperationResponse->getError();
     *         // handleError($error)
     *     }
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string             $firewallPolicy             Name of the firewall policy to update.
     * @param FirewallPolicyRule $firewallPolicyRuleResource The body resource for this request
     * @param string             $project                    Project ID for this request.
     * @param array              $optionalArgs               {
     *     Optional.
     *
     *     @type int $priority
     *           The priority of the rule to patch.
     *     @type string $requestId
     *           An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\OperationResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function patchRule($firewallPolicy, $firewallPolicyRuleResource, $project, array $optionalArgs = [])
    {
        $request = new PatchRuleNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setFirewallPolicyRuleResource($firewallPolicyRuleResource);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['priority'])) {
            $request->setPriority($optionalArgs['priority']);
        }

        if (isset($optionalArgs['requestId'])) {
            $request->setRequestId($optionalArgs['requestId']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startOperationsCall('PatchRule', $optionalArgs, $request, $this->getOperationsClient(), null, Operation::class)->wait();
    }

    /**
     * Removes an association for the specified firewall policy.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $project = 'project';
     *     $operationResponse = $networkFirewallPoliciesClient->removeAssociation($firewallPolicy, $project);
     *     $operationResponse->pollUntilComplete();
     *     if ($operationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $operationResponse->getError();
     *         // handleError($error)
     *     }
     *     // Alternatively:
     *     // start the operation, keep the operation name, and resume later
     *     $operationResponse = $networkFirewallPoliciesClient->removeAssociation($firewallPolicy, $project);
     *     $operationName = $operationResponse->getName();
     *     // ... do other work
     *     $newOperationResponse = $networkFirewallPoliciesClient->resumeOperation($operationName, 'removeAssociation');
     *     while (!$newOperationResponse->isDone()) {
     *         // ... do other work
     *         $newOperationResponse->reload();
     *     }
     *     if ($newOperationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $newOperationResponse->getError();
     *         // handleError($error)
     *     }
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string $firewallPolicy Name of the firewall policy to update.
     * @param string $project        Project ID for this request.
     * @param array  $optionalArgs   {
     *     Optional.
     *
     *     @type string $name
     *           Name for the attachment that will be removed.
     *     @type string $requestId
     *           An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\OperationResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function removeAssociation($firewallPolicy, $project, array $optionalArgs = [])
    {
        $request = new RemoveAssociationNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['name'])) {
            $request->setName($optionalArgs['name']);
        }

        if (isset($optionalArgs['requestId'])) {
            $request->setRequestId($optionalArgs['requestId']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startOperationsCall('RemoveAssociation', $optionalArgs, $request, $this->getOperationsClient(), null, Operation::class)->wait();
    }

    /**
     * Deletes a rule of the specified priority.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $firewallPolicy = 'firewall_policy';
     *     $project = 'project';
     *     $operationResponse = $networkFirewallPoliciesClient->removeRule($firewallPolicy, $project);
     *     $operationResponse->pollUntilComplete();
     *     if ($operationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $operationResponse->getError();
     *         // handleError($error)
     *     }
     *     // Alternatively:
     *     // start the operation, keep the operation name, and resume later
     *     $operationResponse = $networkFirewallPoliciesClient->removeRule($firewallPolicy, $project);
     *     $operationName = $operationResponse->getName();
     *     // ... do other work
     *     $newOperationResponse = $networkFirewallPoliciesClient->resumeOperation($operationName, 'removeRule');
     *     while (!$newOperationResponse->isDone()) {
     *         // ... do other work
     *         $newOperationResponse->reload();
     *     }
     *     if ($newOperationResponse->operationSucceeded()) {
     *         // if creating/modifying, retrieve the target resource
     *     } else {
     *         $error = $newOperationResponse->getError();
     *         // handleError($error)
     *     }
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string $firewallPolicy Name of the firewall policy to update.
     * @param string $project        Project ID for this request.
     * @param array  $optionalArgs   {
     *     Optional.
     *
     *     @type int $priority
     *           The priority of the rule to remove from the firewall policy.
     *     @type string $requestId
     *           An optional request ID to identify requests. Specify a unique request ID so that if you must retry your request, the server will know to ignore the request if it has already been completed. For example, consider a situation where you make an initial request and the request times out. If you make the request again with the same request ID, the server can check if original operation with the same request ID was received, and if so, will ignore the second request. This prevents clients from accidentally creating duplicate commitments. The request ID must be a valid UUID with the exception that zero UUID is not supported ( 00000000-0000-0000-0000-000000000000).
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\OperationResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function removeRule($firewallPolicy, $project, array $optionalArgs = [])
    {
        $request = new RemoveRuleNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setFirewallPolicy($firewallPolicy);
        $request->setProject($project);
        $requestParamHeaders['firewall_policy'] = $firewallPolicy;
        $requestParamHeaders['project'] = $project;
        if (isset($optionalArgs['priority'])) {
            $request->setPriority($optionalArgs['priority']);
        }

        if (isset($optionalArgs['requestId'])) {
            $request->setRequestId($optionalArgs['requestId']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startOperationsCall('RemoveRule', $optionalArgs, $request, $this->getOperationsClient(), null, Operation::class)->wait();
    }

    /**
     * Sets the access control policy on the specified resource. Replaces any existing policy.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $globalSetPolicyRequestResource = new GlobalSetPolicyRequest();
     *     $project = 'project';
     *     $resource = 'resource';
     *     $response = $networkFirewallPoliciesClient->setIamPolicy($globalSetPolicyRequestResource, $project, $resource);
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param GlobalSetPolicyRequest $globalSetPolicyRequestResource The body resource for this request
     * @param string                 $project                        Project ID for this request.
     * @param string                 $resource                       Name or id of the resource for this request.
     * @param array                  $optionalArgs                   {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\Compute\V1\Policy
     *
     * @throws ApiException if the remote call fails
     */
    public function setIamPolicy($globalSetPolicyRequestResource, $project, $resource, array $optionalArgs = [])
    {
        $request = new SetIamPolicyNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setGlobalSetPolicyRequestResource($globalSetPolicyRequestResource);
        $request->setProject($project);
        $request->setResource($resource);
        $requestParamHeaders['project'] = $project;
        $requestParamHeaders['resource'] = $resource;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('SetIamPolicy', Policy::class, $optionalArgs, $request)->wait();
    }

    /**
     * Returns permissions that a caller has on the specified resource.
     *
     * Sample code:
     * ```
     * $networkFirewallPoliciesClient = new NetworkFirewallPoliciesClient();
     * try {
     *     $project = 'project';
     *     $resource = 'resource';
     *     $testPermissionsRequestResource = new TestPermissionsRequest();
     *     $response = $networkFirewallPoliciesClient->testIamPermissions($project, $resource, $testPermissionsRequestResource);
     * } finally {
     *     $networkFirewallPoliciesClient->close();
     * }
     * ```
     *
     * @param string                 $project                        Project ID for this request.
     * @param string                 $resource                       Name or id of the resource for this request.
     * @param TestPermissionsRequest $testPermissionsRequestResource The body resource for this request
     * @param array                  $optionalArgs                   {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\Compute\V1\TestPermissionsResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function testIamPermissions($project, $resource, $testPermissionsRequestResource, array $optionalArgs = [])
    {
        $request = new TestIamPermissionsNetworkFirewallPolicyRequest();
        $requestParamHeaders = [];
        $request->setProject($project);
        $request->setResource($resource);
        $request->setTestPermissionsRequestResource($testPermissionsRequestResource);
        $requestParamHeaders['project'] = $project;
        $requestParamHeaders['resource'] = $resource;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('TestIamPermissions', TestPermissionsResponse::class, $optionalArgs, $request)->wait();
    }
}
