<?php
/*
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * GENERATED CODE WARNING
 * Generated by gapic-generator-php from the file
 * https://github.com/googleapis/googleapis/blob/master/google/cloud/kms/inventory/v1/key_tracking_service.proto
 * Updates to the above are reflected here through a refresh process.
 */

namespace Google\Cloud\Kms\Inventory\V1\Gapic;

use Google\ApiCore\ApiException;
use Google\ApiCore\CredentialsWrapper;
use Google\ApiCore\GapicClientTrait;
use Google\ApiCore\PathTemplate;
use Google\ApiCore\RequestParamsHeaderDescriptor;
use Google\ApiCore\RetrySettings;
use Google\ApiCore\Transport\TransportInterface;
use Google\ApiCore\ValidationException;
use Google\Auth\FetchAuthTokenInterface;
use Google\Cloud\Kms\Inventory\V1\GetProtectedResourcesSummaryRequest;
use Google\Cloud\Kms\Inventory\V1\ProtectedResourcesSummary;
use Google\Cloud\Kms\Inventory\V1\SearchProtectedResourcesRequest;
use Google\Cloud\Kms\Inventory\V1\SearchProtectedResourcesResponse;

/**
 * Service Description: Returns information about the resources in an org that are protected by a
 * given Cloud KMS key via CMEK.
 *
 * This class provides the ability to make remote calls to the backing service through method
 * calls that map to API methods. Sample code to get started:
 *
 * ```
 * $keyTrackingServiceClient = new KeyTrackingServiceClient();
 * try {
 *     $formattedName = $keyTrackingServiceClient->protectedResourcesSummaryName('[PROJECT]', '[LOCATION]', '[KEY_RING]', '[CRYPTO_KEY]');
 *     $response = $keyTrackingServiceClient->getProtectedResourcesSummary($formattedName);
 * } finally {
 *     $keyTrackingServiceClient->close();
 * }
 * ```
 *
 * Many parameters require resource names to be formatted in a particular way. To
 * assist with these names, this class includes a format method for each type of
 * name, and additionally a parseName method to extract the individual identifiers
 * contained within formatted names that are returned by the API.
 */
class KeyTrackingServiceGapicClient
{
    use GapicClientTrait;

    /** The name of the service. */
    const SERVICE_NAME = 'google.cloud.kms.inventory.v1.KeyTrackingService';

    /** The default address of the service. */
    const SERVICE_ADDRESS = 'kmsinventory.googleapis.com';

    /** The default port of the service. */
    const DEFAULT_SERVICE_PORT = 443;

    /** The name of the code generator, to be included in the agent header. */
    const CODEGEN_NAME = 'gapic';

    /** The default scopes required by the service. */
    public static $serviceScopes = [
        'https://www.googleapis.com/auth/cloud-platform',
    ];

    private static $organizationNameTemplate;

    private static $projectLocationKeyRingCryptoKeyCryptoKeyVersionProtectedResourcesSummaryNameTemplate;

    private static $projectLocationKeyRingCryptoKeyProtectedResourcesSummaryNameTemplate;

    private static $protectedResourcesSummaryNameTemplate;

    private static $pathTemplateMap;

    private static function getClientDefaults()
    {
        return [
            'serviceName' => self::SERVICE_NAME,
            'apiEndpoint' =>
                self::SERVICE_ADDRESS . ':' . self::DEFAULT_SERVICE_PORT,
            'clientConfig' =>
                __DIR__ .
                '/../resources/key_tracking_service_client_config.json',
            'descriptorsConfigPath' =>
                __DIR__ .
                '/../resources/key_tracking_service_descriptor_config.php',
            'gcpApiConfigPath' =>
                __DIR__ . '/../resources/key_tracking_service_grpc_config.json',
            'credentialsConfig' => [
                'defaultScopes' => self::$serviceScopes,
            ],
            'transportConfig' => [
                'rest' => [
                    'restClientConfigPath' =>
                        __DIR__ .
                        '/../resources/key_tracking_service_rest_client_config.php',
                ],
            ],
        ];
    }

    private static function getOrganizationNameTemplate()
    {
        if (self::$organizationNameTemplate == null) {
            self::$organizationNameTemplate = new PathTemplate(
                'organizations/{organization}'
            );
        }

        return self::$organizationNameTemplate;
    }

    private static function getProjectLocationKeyRingCryptoKeyCryptoKeyVersionProtectedResourcesSummaryNameTemplate()
    {
        if (
            self::$projectLocationKeyRingCryptoKeyCryptoKeyVersionProtectedResourcesSummaryNameTemplate ==
            null
        ) {
            self::$projectLocationKeyRingCryptoKeyCryptoKeyVersionProtectedResourcesSummaryNameTemplate = new PathTemplate(
                'projects/{project}/locations/{location}/keyRings/{key_ring}/cryptoKeys/{crypto_key}/cryptoKeyVersions/{crypto_key_version}/protectedResourcesSummary'
            );
        }

        return self::$projectLocationKeyRingCryptoKeyCryptoKeyVersionProtectedResourcesSummaryNameTemplate;
    }

    private static function getProjectLocationKeyRingCryptoKeyProtectedResourcesSummaryNameTemplate()
    {
        if (
            self::$projectLocationKeyRingCryptoKeyProtectedResourcesSummaryNameTemplate ==
            null
        ) {
            self::$projectLocationKeyRingCryptoKeyProtectedResourcesSummaryNameTemplate = new PathTemplate(
                'projects/{project}/locations/{location}/keyRings/{key_ring}/cryptoKeys/{crypto_key}/protectedResourcesSummary'
            );
        }

        return self::$projectLocationKeyRingCryptoKeyProtectedResourcesSummaryNameTemplate;
    }

    private static function getProtectedResourcesSummaryNameTemplate()
    {
        if (self::$protectedResourcesSummaryNameTemplate == null) {
            self::$protectedResourcesSummaryNameTemplate = new PathTemplate(
                'projects/{project}/locations/{location}/keyRings/{key_ring}/cryptoKeys/{crypto_key}/protectedResourcesSummary'
            );
        }

        return self::$protectedResourcesSummaryNameTemplate;
    }

    private static function getPathTemplateMap()
    {
        if (self::$pathTemplateMap == null) {
            self::$pathTemplateMap = [
                'organization' => self::getOrganizationNameTemplate(),
                'projectLocationKeyRingCryptoKeyCryptoKeyVersionProtectedResourcesSummary' => self::getProjectLocationKeyRingCryptoKeyCryptoKeyVersionProtectedResourcesSummaryNameTemplate(),
                'projectLocationKeyRingCryptoKeyProtectedResourcesSummary' => self::getProjectLocationKeyRingCryptoKeyProtectedResourcesSummaryNameTemplate(),
                'protectedResourcesSummary' => self::getProtectedResourcesSummaryNameTemplate(),
            ];
        }

        return self::$pathTemplateMap;
    }

    /**
     * Formats a string containing the fully-qualified path to represent a organization
     * resource.
     *
     * @param string $organization
     *
     * @return string The formatted organization resource.
     */
    public static function organizationName($organization)
    {
        return self::getOrganizationNameTemplate()->render([
            'organization' => $organization,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * project_location_key_ring_crypto_key_crypto_key_version_protectedResourcesSummary
     * resource.
     *
     * @param string $project
     * @param string $location
     * @param string $keyRing
     * @param string $cryptoKey
     * @param string $cryptoKeyVersion
     *
     * @return string The formatted project_location_key_ring_crypto_key_crypto_key_version_protectedResourcesSummary resource.
     */
    public static function projectLocationKeyRingCryptoKeyCryptoKeyVersionProtectedResourcesSummaryName(
        $project,
        $location,
        $keyRing,
        $cryptoKey,
        $cryptoKeyVersion
    ) {
        return self::getProjectLocationKeyRingCryptoKeyCryptoKeyVersionProtectedResourcesSummaryNameTemplate()->render(
            [
                'project' => $project,
                'location' => $location,
                'key_ring' => $keyRing,
                'crypto_key' => $cryptoKey,
                'crypto_key_version' => $cryptoKeyVersion,
            ]
        );
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * project_location_key_ring_crypto_key_protectedResourcesSummary resource.
     *
     * @param string $project
     * @param string $location
     * @param string $keyRing
     * @param string $cryptoKey
     *
     * @return string The formatted project_location_key_ring_crypto_key_protectedResourcesSummary resource.
     */
    public static function projectLocationKeyRingCryptoKeyProtectedResourcesSummaryName(
        $project,
        $location,
        $keyRing,
        $cryptoKey
    ) {
        return self::getProjectLocationKeyRingCryptoKeyProtectedResourcesSummaryNameTemplate()->render(
            [
                'project' => $project,
                'location' => $location,
                'key_ring' => $keyRing,
                'crypto_key' => $cryptoKey,
            ]
        );
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * protected_resources_summary resource.
     *
     * @param string $project
     * @param string $location
     * @param string $keyRing
     * @param string $cryptoKey
     *
     * @return string The formatted protected_resources_summary resource.
     */
    public static function protectedResourcesSummaryName(
        $project,
        $location,
        $keyRing,
        $cryptoKey
    ) {
        return self::getProtectedResourcesSummaryNameTemplate()->render([
            'project' => $project,
            'location' => $location,
            'key_ring' => $keyRing,
            'crypto_key' => $cryptoKey,
        ]);
    }

    /**
     * Parses a formatted name string and returns an associative array of the components in the name.
     * The following name formats are supported:
     * Template: Pattern
     * - organization: organizations/{organization}
     * - projectLocationKeyRingCryptoKeyCryptoKeyVersionProtectedResourcesSummary: projects/{project}/locations/{location}/keyRings/{key_ring}/cryptoKeys/{crypto_key}/cryptoKeyVersions/{crypto_key_version}/protectedResourcesSummary
     * - projectLocationKeyRingCryptoKeyProtectedResourcesSummary: projects/{project}/locations/{location}/keyRings/{key_ring}/cryptoKeys/{crypto_key}/protectedResourcesSummary
     * - protectedResourcesSummary: projects/{project}/locations/{location}/keyRings/{key_ring}/cryptoKeys/{crypto_key}/protectedResourcesSummary
     *
     * The optional $template argument can be supplied to specify a particular pattern,
     * and must match one of the templates listed above. If no $template argument is
     * provided, or if the $template argument does not match one of the templates
     * listed, then parseName will check each of the supported templates, and return
     * the first match.
     *
     * @param string $formattedName The formatted name string
     * @param string $template      Optional name of template to match
     *
     * @return array An associative array from name component IDs to component values.
     *
     * @throws ValidationException If $formattedName could not be matched.
     */
    public static function parseName($formattedName, $template = null)
    {
        $templateMap = self::getPathTemplateMap();
        if ($template) {
            if (!isset($templateMap[$template])) {
                throw new ValidationException(
                    "Template name $template does not exist"
                );
            }

            return $templateMap[$template]->match($formattedName);
        }

        foreach ($templateMap as $templateName => $pathTemplate) {
            try {
                return $pathTemplate->match($formattedName);
            } catch (ValidationException $ex) {
                // Swallow the exception to continue trying other path templates
            }
        }

        throw new ValidationException(
            "Input did not match any known format. Input: $formattedName"
        );
    }

    /**
     * Constructor.
     *
     * @param array $options {
     *     Optional. Options for configuring the service API wrapper.
     *
     *     @type string $apiEndpoint
     *           The address of the API remote host. May optionally include the port, formatted
     *           as "<uri>:<port>". Default 'kmsinventory.googleapis.com:443'.
     *     @type string|array|FetchAuthTokenInterface|CredentialsWrapper $credentials
     *           The credentials to be used by the client to authorize API calls. This option
     *           accepts either a path to a credentials file, or a decoded credentials file as a
     *           PHP array.
     *           *Advanced usage*: In addition, this option can also accept a pre-constructed
     *           {@see \Google\Auth\FetchAuthTokenInterface} object or
     *           {@see \Google\ApiCore\CredentialsWrapper} object. Note that when one of these
     *           objects are provided, any settings in $credentialsConfig will be ignored.
     *     @type array $credentialsConfig
     *           Options used to configure credentials, including auth token caching, for the
     *           client. For a full list of supporting configuration options, see
     *           {@see \Google\ApiCore\CredentialsWrapper::build()} .
     *     @type bool $disableRetries
     *           Determines whether or not retries defined by the client configuration should be
     *           disabled. Defaults to `false`.
     *     @type string|array $clientConfig
     *           Client method configuration, including retry settings. This option can be either
     *           a path to a JSON file, or a PHP array containing the decoded JSON data. By
     *           default this settings points to the default client config file, which is
     *           provided in the resources folder.
     *     @type string|TransportInterface $transport
     *           The transport used for executing network requests. May be either the string
     *           `rest` or `grpc`. Defaults to `grpc` if gRPC support is detected on the system.
     *           *Advanced usage*: Additionally, it is possible to pass in an already
     *           instantiated {@see \Google\ApiCore\Transport\TransportInterface} object. Note
     *           that when this object is provided, any settings in $transportConfig, and any
     *           $apiEndpoint setting, will be ignored.
     *     @type array $transportConfig
     *           Configuration options that will be used to construct the transport. Options for
     *           each supported transport type should be passed in a key for that transport. For
     *           example:
     *           $transportConfig = [
     *               'grpc' => [...],
     *               'rest' => [...],
     *           ];
     *           See the {@see \Google\ApiCore\Transport\GrpcTransport::build()} and
     *           {@see \Google\ApiCore\Transport\RestTransport::build()} methods for the
     *           supported options.
     *     @type callable $clientCertSource
     *           A callable which returns the client cert as a string. This can be used to
     *           provide a certificate and private key to the transport layer for mTLS.
     * }
     *
     * @throws ValidationException
     */
    public function __construct(array $options = [])
    {
        $clientOptions = $this->buildClientOptions($options);
        $this->setClientOptions($clientOptions);
    }

    /**
     * Returns aggregate information about the resources protected by the given
     * Cloud KMS [CryptoKey][google.cloud.kms.v1.CryptoKey]. Only resources within
     * the same Cloud organization as the key will be returned. The project that
     * holds the key must be part of an organization in order for this call to
     * succeed.
     *
     * Sample code:
     * ```
     * $keyTrackingServiceClient = new KeyTrackingServiceClient();
     * try {
     *     $formattedName = $keyTrackingServiceClient->protectedResourcesSummaryName('[PROJECT]', '[LOCATION]', '[KEY_RING]', '[CRYPTO_KEY]');
     *     $response = $keyTrackingServiceClient->getProtectedResourcesSummary($formattedName);
     * } finally {
     *     $keyTrackingServiceClient->close();
     * }
     * ```
     *
     * @param string $name         Required. The resource name of the
     *                             [CryptoKey][google.cloud.kms.v1.CryptoKey].
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\Kms\Inventory\V1\ProtectedResourcesSummary
     *
     * @throws ApiException if the remote call fails
     */
    public function getProtectedResourcesSummary(
        $name,
        array $optionalArgs = []
    ) {
        $request = new GetProtectedResourcesSummaryRequest();
        $requestParamHeaders = [];
        $request->setName($name);
        $requestParamHeaders['name'] = $name;
        $requestParams = new RequestParamsHeaderDescriptor(
            $requestParamHeaders
        );
        $optionalArgs['headers'] = isset($optionalArgs['headers'])
            ? array_merge($requestParams->getHeader(), $optionalArgs['headers'])
            : $requestParams->getHeader();
        return $this->startCall(
            'GetProtectedResourcesSummary',
            ProtectedResourcesSummary::class,
            $optionalArgs,
            $request
        )->wait();
    }

    /**
     * Returns metadata about the resources protected by the given Cloud KMS
     * [CryptoKey][google.cloud.kms.v1.CryptoKey] in the given Cloud organization.
     *
     * Sample code:
     * ```
     * $keyTrackingServiceClient = new KeyTrackingServiceClient();
     * try {
     *     $formattedScope = $keyTrackingServiceClient->organizationName('[ORGANIZATION]');
     *     $cryptoKey = 'crypto_key';
     *     // Iterate over pages of elements
     *     $pagedResponse = $keyTrackingServiceClient->searchProtectedResources($formattedScope, $cryptoKey);
     *     foreach ($pagedResponse->iteratePages() as $page) {
     *         foreach ($page as $element) {
     *             // doSomethingWith($element);
     *         }
     *     }
     *     // Alternatively:
     *     // Iterate through all elements
     *     $pagedResponse = $keyTrackingServiceClient->searchProtectedResources($formattedScope, $cryptoKey);
     *     foreach ($pagedResponse->iterateAllElements() as $element) {
     *         // doSomethingWith($element);
     *     }
     * } finally {
     *     $keyTrackingServiceClient->close();
     * }
     * ```
     *
     * @param string $scope        Required. Resource name of the organization.
     *                             Example: organizations/123
     * @param string $cryptoKey    Required. The resource name of the
     *                             [CryptoKey][google.cloud.kms.v1.CryptoKey].
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type int $pageSize
     *           The maximum number of resources contained in the underlying API
     *           response. The API may return fewer values in a page, even if
     *           there are additional values to be retrieved.
     *     @type string $pageToken
     *           A page token is used to specify a page of values to be returned.
     *           If no page token is specified (the default), the first page
     *           of values will be returned. Any page token used here must have
     *           been generated by a previous call to the API.
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\PagedListResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function searchProtectedResources(
        $scope,
        $cryptoKey,
        array $optionalArgs = []
    ) {
        $request = new SearchProtectedResourcesRequest();
        $requestParamHeaders = [];
        $request->setScope($scope);
        $request->setCryptoKey($cryptoKey);
        $requestParamHeaders['scope'] = $scope;
        if (isset($optionalArgs['pageSize'])) {
            $request->setPageSize($optionalArgs['pageSize']);
        }

        if (isset($optionalArgs['pageToken'])) {
            $request->setPageToken($optionalArgs['pageToken']);
        }

        $requestParams = new RequestParamsHeaderDescriptor(
            $requestParamHeaders
        );
        $optionalArgs['headers'] = isset($optionalArgs['headers'])
            ? array_merge($requestParams->getHeader(), $optionalArgs['headers'])
            : $requestParams->getHeader();
        return $this->getPagedListResponse(
            'SearchProtectedResources',
            $optionalArgs,
            SearchProtectedResourcesResponse::class,
            $request
        );
    }
}
